/* Filesystem storage types. */

#pragma once

#include <stdint.h>
#include <sys/types.h>

/* File-based addressing. */

typedef size_t location_t;        /* file locations and sizes */

/* Specific storage representation choices. */

typedef uint16_t length_t;        /* lengths of directory entries */
typedef uint16_t owner_t;         /* owner data */
typedef uint16_t permissions_t;   /* permissions data */

typedef enum
{
  OBJTYPE_DIR = 0,                /* directory */
  OBJTYPE_FILE = 1,               /* file */
} objtype_t;

typedef struct
{
  location_t size;
  objtype_t type;
  owner_t owner;
  permissions_t permissions;
} metadata_t;

typedef union
{
  const char *name;             /* input string (for querying, updating) */
  char namearray[1];            /* stored string (first character) */
} entry_name_t;

typedef struct
{
  length_t length;                /* length of entry */
  location_t location;            /* location of file/directory */
  metadata_t metadata;            /* object metadata */
  entry_name_t n;
} entry_t;
